<?php

/**
 * @author info@senbyte.com
 * @copyright 2016 senByte UG
 * @license
 */

require_once PFAD_ROOT . PFAD_INCLUDES_MODULES . 'PaymentMethod.class.php';
require_once str_replace('frontend/', '', $oPlugin->cFrontendPfad) . 'easycredit-sdk/autoload.php';
require_once str_replace('frontend/', '', $oPlugin->cFrontendPfad) . 'paymentmethod/EasyCreditProcess.php';
include_once $oPlugin->cAdminmenuPfad . 'inc/easyCreditDefinitions.php';

use EasyCredit\Api\ApiClient                 as ecClient;
use EasyCredit\Api\DataMapper                as ecMapper;
use EasyCredit\Config                        as ecConfig;
use EasyCredit\Transfer\PersonData           as ecPerson;
use EasyCredit\Transfer\BillingAddress       as ecBillingAddress;
use EasyCredit\Transfer\DeliveryAddress      as ecDeliveryAddress;
use EasyCredit\Transfer\AdditionalPersonData as ecAdditionalData;
use EasyCredit\Transfer\CartInfoCollection   as ecCartCollection;
use EasyCredit\Transfer\CartInfo             as ecCartItem;
use EasyCredit\Transfer\CallbackUrls         as ecCallbackUrls;
use EasyCredit\Transfer\ArticleIdCollection  as ecArticleIdCollection;
use EasyCredit\Transfer\ArticleId            as ecArticleId;
use EasyCredit\Log\Logger                    as ecLogger;
use EasyCredit\Log\Handler\FileHandler       as ecFileHandler;
use EasyCredit\Transfer\InstallmentPlan;

class EasyCredit extends PaymentMethod
{
    /**
     * @var Plugin
     */
    public $oPlugin;

    /**
     * @var string
     */
    public $pluginName;

    /**
     * @var string
     */
    public $webshopId;
     
    /**
     * @var string
     */
    public $token;

    /**
     * @var Request
     */
    public $request;

    /**
     *
     * @var ApiClient
     */
    public $apiClient;
    
    /**
     * @var Process
     */
    protected $ecProcess;
    
    /**
     * @var bool
     */
    protected $selectable;
    
    static $TITLE_VALUES = array("PROFDR", "DR", "PROF", "DRDR");

    public function __construct()
    {
        $plugin             = Plugin::getPluginById('sb_jtl_easycredit');
        $oPlugin            = new Plugin($plugin->kPlugin);
        $this->oPlugin      = $oPlugin;
        $this->pluginName   = 'kPlugin_' . $oPlugin->kPlugin . '_ratenkaufbyeasycredit';
        $this->webshopId    = $oPlugin->oPluginEinstellungAssoc_arr[$this->pluginName . '_webshop_id'];
        $this->token        = $oPlugin->oPluginEinstellungAssoc_arr[$this->pluginName . '_webshop_token'];
        $this->selectable   = true;


        $loggingEnabled = $oPlugin->oPluginEinstellungAssoc_arr['jtl_easycredit_logging_enable'];
        $adapter        = new \EasyCredit\Http\Adapter\Curl();

        if ($loggingEnabled == "1") {
            $logLevel    = $oPlugin->oPluginEinstellungAssoc_arr['jtl_easycredit_logging_level'];
            $logPath     = str_replace('frontend/', '', $oPlugin->cFrontendPfad) . 'log/EasyCredit_' . date("Y-m-d") . '.log';
            $fileHandler = new ecFileHandler($logPath);
            $logger      = new ecLogger($fileHandler, $logLevel);
            $adapter->setLogger($logger);
        }

        $this->request = new EasyCredit\Http\Request(
            \EasyCredit\Config::EASYCREDIT_API_HOSTNAME,
            \EasyCredit\Config::EASYCREDIT_API_PORT,
            $adapter
            );

        $this->apiClient = new ecClient(
            $this->webshopId,
            $this->token,
            $this->request,
            new ecMapper()
            );

        try {
            $this->ecProcess = EasyCreditProcess::createInstance($this->apiClient);
            $this->doLog("EasyCredit process instance created", LOGLEVEL_DEBUG);
        } catch (Exception $e) {
            $this->doLog("EasyCredit process instance could not be created! will destroy and re-create instance", LOGLEVEL_ERROR);
            $this->ecProcess = EasyCreditProcess::getInstance();
        }

        parent::__construct($this->pluginName);
    }

    /**
     * @param Bestellung $order
     */
    public function preparePaymentProcess($order)
    {
        $hash = $this->generateHash($order);
        header('Location: '.$this->getNotificationURL($hash));
    }
    
    /**
     * @param Warenkorb $basket
     */
    public function startPaymentProcess($basket)
    {
        $this->doLog("Preparing Payment Process...", LOGLEVEL_DEBUG);
        $this->ecProcess->getProcessData()->initEmpty();
        $this->initProcess($basket);
        $this->ecProcess->getProcessData()->save();

        $processData = $this->ecProcess->getProcessData();

        $this->doLog("Process status: " . $processData->getStatus(), LOGLEVEL_DEBUG);
        
        if ($processData->getStatus() === \EasyCredit\Process\Status::NONE) {
            try {
                $this->ecProcess->initialize();
                $this->doLog(
                    "Process initialized with TBA: " . 
                    $this->ecProcess->getProcessData()->getTbaId(),
                    LOGLEVEL_DEBUG
                    );
            } catch (\EasyCredit\Process\Exception\InvalidTransitionException $e) {
                $this->doLog("Process initialization failed!", LOGLEVEL_ERROR);
            }
        }
                
        if ($this->ecProcess->getProcessData()->getTbaId()) {
            $redirect = sprintf(ecConfig::PAYMENT_PAGE_URL, $this->ecProcess->getProcessData()->getTbaId());
        } else {
            $msg = "";
            foreach ($this->ecProcess->getProcessData()->getMessages() as $key => $message) {
                if (!empty($msg)) {
                    $msg.='<br>';
                }
                $msg.= $message;
            }
            $_SESSION['ec_error_msg'] = utf8_decode($msg);
            $redirect                 = 'bestellvorgang.php?editRechnungsadresse=1';
            $this->doLog("Messages received from EasyCredit: " . $_SESSION['ec_error_msg'], LOGLEVEL_DEBUG);
            $this->unsetSessionParams();
        }
        $this->doLog("Redirecting to: " . $redirect, LOGLEVEL_DEBUG);
        
        header('Location: '.$redirect);
    }

    /**
     * @param Bestellung $order
     * @param string     $paymentHash
     * @param array      $args
     *
     * @return bool
     */
    public function handleNotification($order, $paymentHash, $args)
    {
        if ($this->verifyNotification($order, $paymentHash, $args)) {
            $helper = new BestellungHelper($order);
            $result = $this->ecProcess->agree();
            if ($result) {
                $this->doLog(
                    "confirmed order to EasyCredit for TBA: "
                    . $this->ecProcess->getProcessData()->getTbaId(),
                    LOGLEVEL_DEBUG
                    );

                $this->addIncomingPayment($order, [
                    'fBetrag'          => $order->fGesamtsumme,
                    'fZahlungsgebuehr' => $helper->getTotal()->surcharge[WarenkorbHelper::GROSS],
                    'cISO'             => $order->Waehrung->cISO,
                    'cHinweis'         => $this->ecProcess->getProcessData()->getTechnicalTbaId(),
                ]);

                $this->doLog(
                    "Payment added for TBA: "
                    . $this->ecProcess->getProcessData()->getTbaId(),
                    LOGLEVEL_DEBUG
                    );

                $this->setOrderStatusToPaid($order);

                $this->doLog(
                    "order status set to paid for TBA: "
                    . $this->ecProcess->getProcessData()->getTbaId(),
                    LOGLEVEL_DEBUG
                    );
                
                $this->ecProcess->destroy();
                $this->unsetSessionParams();
            }
        } else {
            $result = false;

            $this->doLog(
                "Confirmation to EasyCredit failed for TBA: "
                . $this->ecProcess->getProcessData()->getTbaId(),
                LOGLEVEL_DEBUG
                );
        }

        return $result;
    }

    /**
     * @param Bestellung $order
     * @param string     $hash
     * @param array      $args
     *
     * @return true, if $order should be finalized
     */
    public function finalizeOrder($order, $hash, $args)
    {
        if ($this->verifyNotification($order, $hash, $args)) {
            $this->doLog(
                "setting process status to SAVED for TBA: "
                . $this->ecProcess->getProcessData()->getTbaId(),
                LOGLEVEL_DEBUG
                );

            $this->ecProcess->getProcessData()->setStatus('SAVED');
            
            $decision = $this->ecProcess->getDecision();
            $result   = $decision->getDecision()->getResult() == 'GRUEN' ? true : false;
            
            $this->doLog(
                "Received " . $result ? "positive" : "negative" .  " result for TBA: "
                . $this->ecProcess->getProcessData()->getTbaId(),
                LOGLEVEL_DEBUG
                );
        } else {
            $result = false;
            $this->unsetSessionParams();
        }

        return $result;
    }

    /**
     *
     * @param Bestellung $order
     * @param string     $paymentHash
     * @param array      $args
     *
     * @return bool
     */
    function verifyNotification($order, $paymentHash, $args)
    {
        $jtlTotal = $order->fGesamtsumme;
        $ecTotal  = $this->ecProcess->getProcessData()->getOrderTotal();

        if (!$jtlTotal == $ecTotal) {
            $this->doLog(
                "Order Totals verification failed for TBA: "
                . $this->ecProcess->getProcessData()->getTbaId()
                . " JTL Order total " . $jtlTotal . "does NOT match"
                . " EasyCredit order total " . $ecTotal,
                LOGLEVEL_DEBUG
                );
            
            return false;
        } else {
            $this->doLog(
                "Order Totals verified successfully for TBA: "
                . $this->ecProcess->getProcessData()->getTbaId(),
                LOGLEVEL_DEBUG
                );

            return true;
        }
    }

    /**
     * @param Warenkorb $basket
     */
    protected function initProcess($basket)
    {
        header('Content-Type: text/html; charset=UTF-8');
        $basketHelper = new WarenkorbHelper($basket);
        
        $this->ecProcess->getProcessData()->setOrderTotal(floatval($basketHelper->getTotal(2)->total[1]));
        $this->ecProcess->getProcessData()->setTerm(intval($this->getUTF8String($_SESSION['ecipt'])));

        //Personal Data
        $customer   = $basketHelper->getCustomer();
        $ecCustomer = $this->ecProcess->getProcessData()->getCustomer();
        $ecPerson   = $this->ecProcess->getProcessData()->getCustomer()->getPersonData();
        $ecPerson->setFirstName($this->getUTF8String($customer->cVorname));
        $ecPerson->setLastName($this->getUTF8String($customer->cNachname));
        $ecPerson->setSalutation(
            $this->getUTF8String($customer->cAnrede == 'm' ? ecPerson::SALUTATION_MR : ecPerson::SALUTATION_MRS)
            );
        if ($this->is_date($customer->dGeburtstag)) {
            $ecPerson->setBirthDate($customer->dGeburtstag);
        }
        $ecCustomer->setPersonData($ecPerson);

        //Billing Address
        $billingAddress = $basketHelper->getCustomer();
        
        $ecBilling      = new ecBillingAddress();

        $ecBilling->setStreet($this->getUTF8String($billingAddress->cStrasse . ' ' . $billingAddress->cHausnummer));
        $ecBilling->setAddressAdditional($this->getUTF8String($billingAddress->cAdressZusatz));
        $ecBilling->setZip($this->getUTF8String($billingAddress->cPLZ));
        $ecBilling->setCity($this->getUTF8String($billingAddress->cOrt));
        $ecBilling->setCountryCode($this->getUTF8String($billingAddress->cLand));
        $this->ecProcess->getProcessData()->setBillingAddress($ecBilling);

        //alternative Delivery Address if present
        $deliveryAddress = $basketHelper->getShippingAddress();
        $ecDelivery      = new ecDeliveryAddress();

        $ecDelivery->setStreet($this->getUTF8String($deliveryAddress->cStrasse . ' ' . $deliveryAddress->cHausnummer));
        $ecDelivery->setAddressAdditional($this->getUTF8String($deliveryAddress->cAdressZusatz));
        $ecDelivery->setZip($this->getUTF8String($deliveryAddress->cPLZ));
        $ecDelivery->setCity($this->getUTF8String($deliveryAddress->cOrt));
        $ecDelivery->setCountryCode($this->getUTF8String($deliveryAddress->cLand));
        $this->ecProcess->getProcessData()->setDeliveryAddress($ecDelivery);

        // contact data
        $ecCustomer->getContact()->setEmail($this->getUTF8String($billingAddress->cMail));
        
        if (!empty($billingAddress->cMobil)) {
            $ecCustomer->getContact()->setMobilphoneVerify(true);
            $ecCustomer->getContact()->setMobilphone($this->getUTF8String($billingAddress->cMobil));
        } else {
            $ecCustomer->getContact()->setMobilphoneVerify(false);
        }

        if (!empty($customer->cTitel) || !empty($billingAddress->cTel)) {
            // additional person data
            $ecAdditional = new ecAdditionalData();
            if (!empty($customer->cTel)) {
                $ecAdditional->setPhoneNumber($billingAddress->cTel);
            }

            if (!empty($customer->cTitel)) {
                $ecAdditional->setTitle($this->matchTitle($billingAddress->cTitel));
            }

            $ecCustomer->setAdditionalPersonData($ecAdditional);
        }
        $this->ecProcess->getProcessData()->setCustomer($ecCustomer);

        //Risk related info
        $riskRelatedInfo = new \EasyCredit\Transfer\RiskRelatedInfo();

        $riskRelatedInfo->setNegativePaymentInformation(
            \EasyCredit\Transfer\RiskRelatedInfo::NEGATIVE_PAYMENT_INFORMATION_NO_INFORMATION
            );

        
        if (!$customer->nRegistriert) {
            $riskRelatedInfo->setCustomerRegistrated(false);
        } else {
            $riskRelatedInfo->setCustomerRegistrated(true);

            $customerRegistratedAt = new \DateTime($customer->dErstellt);
            $riskRelatedInfo->setCustomerRegistrationDate($customerRegistratedAt);

            $orderCount   = $customer->kKunde ?
            $GLOBALS['DB']->executeQuery("select count(*) as nr from tbestellung where kKunde=" . $customer->kKunde , 1)->nr
            : 0;

            $riskRelatedInfo->setOrderCount(intval($orderCount));
        }
        $riskRelatedInfo->setRiskItemInCart(false);
        
        $riskRelatedInfo->setCartItemsCount(count($basketHelper->getObject()->PositionenArr));
        $this->ecProcess->getProcessData()->setRiskInfo($riskRelatedInfo);

        //Warenkorb
        $ecCartCollection = new ecCartCollection();
        $cartItems        = $basketHelper->getObject()->PositionenArr;

        foreach ($cartItems as $item) {
            if (null !== $item->Artikel) {
                $ecCartItem = new ecCartItem();
                $ecCartItem->setName($this->getUTF8String($item->Artikel->cName));
                $ecCartItem->setQuantity(intval($item->nAnzahl));
        
                $val = explode(' ', array_values($item->cGesamtpreisLocalized[0])[0]);
        
                $from      = array('.', ' &euro;', ',');
                $to        = array('', '', '.');
                $itemPrice = str_replace($from, $to, $val[0]);
        
                $ecCartItem->setPrice(intval($itemPrice));
        
                $skuCollection = new ecArticleIdCollection();
                $sku           = new ecArticleId();
        
                $sku->setType(ecArticleId::TYPE_SKU);
                $sku->setId($this->getUTF8String($item->cArtNr));
                $skuCollection->addItem($sku);
        
                $ecCartItem->setArticleId($skuCollection);
                $ecCartItem->setManufacture($this->getUTF8String($item->Artikel->cHersteller));
        
                $ecCartCollection->addItem($ecCartItem);
            }
        }
        
        $this->ecProcess->getProcessData()->setProducts($ecCartCollection);

        $callbackUrls = new ecCallbackUrls();

        $callbackUrls->setUrlSucceeded(Shop::getUrl(true) . '/bestellvorgang.php');
        $callbackUrls->setUrlCancelled(Shop::getUrl(true) . '/bestellvorgang.php?editZahlungsart=1');
        $callbackUrls->setUrlDenied(Shop::getUrl(true) . '/bestellvorgang.php?editZahlungsart=1&ec_dec=1');
        $this->ecProcess->getProcessData()->setCallbackUrls($callbackUrls);
    }

    /**
     * @param Bestellung $order
     *
     * @return boolean
     */
    protected function isAternativeDelivery($order)
    {
        return $order->oRechnungsadresse === $order->Lieferadresse ? false : true;
    }

    /**
     * @param string $title
     *
     * @return string|null $value
     */
    protected function matchTitle($title) {
        $prepared = strtoupper(preg_replace("/[^a-zA-Z]/", "", $title));

        foreach (self::$TITLE_VALUES as $value) {
            if(strcasecmp($prepared, $value) == 0) {
                return $value;
                break;
            }
        }

        return null;
    }

    /**
     * @param string $string
     *
     * @return string
     */
    protected function getUTF8String($string) {
        return utf8_encode(
            html_entity_decode($string)
            );
    }

    /**
     * @return bool
     */
    public function redirectOnPaymentSuccess()
    {
        return true;
    }

    /**
     * @return bool
     */
    public function redirectOnCancel()
    {
        return true;
    }

    /**
     * @return bool
     */
    public function canPayAgain()
    {
        return false;
    }
    
    /**
     * @return Process
     */
    public function getEcProcess() {
        return $this->ecProcess;
    }
    
    /**
     * @param unknown $price
     * @param unknown $numberOfRates
     * 
     * @return InstallmentPlan
     */
    public function getModelInstallmentPlan($price, $numberOfRates)
    {
        $modelCalculation = $this->ecProcess->getModelCalculation($price);
        $results          = $modelCalculation->getResults();
        foreach ($results as $n => $installmentPlan) {
            if($installmentPlan->getPaymentSchedule()->getNumberOfRates() == $numberOfRates) {
                return $installmentPlan;
            } else {
                continue;
            }
        }
        return $results->offSetGet($results->count()-1);
    }
    
    /**
     * @param array $aPost_arr
     * 
     * @return bool
     */
    public function handleAdditional($aPost_arr)
    {
        if ($this->validateAdditional()) {
            $this->startPaymentProcess(Session::Basket());
            return true;
        } else {
            return false;
        }
    }
    
    /**
     * @return bool
     */
    public function validateAdditional()
    {
        if (Session::get('ecipt') && Session::get('ecct') == 1) {
            return true;
        } else {
            return false;
        }
    }
    
    /**
     * @return bool
     */
    public function isSelectable()
    {
        $customer = Session::get('Kunde');
        $country  = $customer->cLand;
        
        if($country != 'DE') {
            $this->selectable = false;
        } else {
            $purchaseLimitResponse = $this->apiClient->getResidualAmountPurchaseLimit();
            $basket                = Session::get('Warenkorb');
            $basketHelper          = new WarenkorbHelper($basket);
            $ordertotal            = floatval($basketHelper->getTotal(2)->total[1]);
            
            if($purchaseLimitResponse->getResidualAmount() < $ordertotal) {
                $this->selectable = false;
            } else {
                $this->selectable = true;
            }
        }
        
        return $this->selectable;
    }
    
    /**
     * @param bool $selectable
     */
    public function setSelectable($selectable)
    {
        $this->selectable = $selectable;
    }
    
    /**
     * removes easyCredit specific session parameters
     */
    protected function unsetSessionParams() {
        unset($_SESSION['ecipt']);
        unset($_SESSION['ecct']);
    }
    
    /**
     * @param string $str
     * 
     * @return boolean
     */
    protected function is_date( $str ) {
        try {
            $dt = new DateTime( trim($str) );
        }
        catch( Exception $e ) {
            return false;
        }

        $month = $dt->format('m');
        $day   = $dt->format('d');
        $year  = $dt->format('Y');

        if( checkdate($month, $day, $year) ) {
            return true;
        }
        else {
            return false;
        }
    }
}
